'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const qs = require("query-string");
const types_1 = require("./types");
const util_1 = require("util");
class LyticsClientOptions {
    constructor(apikey) {
        this.base_url = 'https://api.lytics.io';
        this.apikey = apikey;
    }
}
exports.LyticsClientOptions = LyticsClientOptions;
class LyticsClient {
    constructor(apikey) {
        this.base_url = 'https://api.lytics.io';
        this.apikey = apikey;
        this.headers = {
            'Authorization': this.apikey,
            'Content-Type': 'application/json'
        };
    }
    static create(options) {
        const client = new LyticsClient(options.apikey);
        client.base_url = options.base_url;
        return client;
    }
    doRequest(config, dataHandler) {
        return __awaiter(this, void 0, void 0, function* () {
            var wasSuccess = false;
            const response = yield axios_1.default.request(config);
            const data = response.data;
            if (!data && response.status === 204) {
                return Promise.resolve(undefined);
            }
            if (data.status === 200 || data.status === 201) {
                if (data.message === 'Not Found') {
                    return Promise.resolve(undefined);
                }
                if (data.message === 'created' || data.message === 'success' || data.message === 'updated' || data.message.length === 0) {
                    wasSuccess = true;
                }
            }
            if (!wasSuccess && data.status === 'success') {
                wasSuccess = true;
            }
            if (wasSuccess) {
                const data2 = dataHandler ? dataHandler(data) : data.data;
                return Promise.resolve(data2);
            }
            return Promise.reject(data);
        });
    }
    doGet(url, dataHandler) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = {
                url: url,
                method: 'get',
                headers: this.headers
            };
            return this.doRequest(config, dataHandler);
        });
    }
    doPost(url, data, dataHandler) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof data === "object") {
                data = JSON.stringify(data);
            }
            const config = {
                url: url,
                method: 'post',
                data: data,
                headers: this.headers
            };
            return this.doRequest(config, dataHandler);
        });
    }
    doPut(url, data, dataHandler) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof data === "object") {
                data = JSON.stringify(data);
            }
            const config = {
                url: url,
                method: 'put',
                data: data,
                headers: this.headers
            };
            return this.doRequest(config, dataHandler);
        });
    }
    doDelete(url, dataHandler) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = {
                url: url,
                method: 'delete',
                headers: this.headers
            };
            return this.doRequest(config, dataHandler);
        });
    }
    getAccounts() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/account`;
            return this.doGet(url);
        });
    }
    getAccount(aid) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/account/${aid}`;
            return this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(undefined);
                }
                throw err;
            });
        });
    }
    getStreams() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/schema/_streams`;
            return this.doGet(url);
        });
    }
    getStream(name) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(name)) {
                throw new Error('Required parameter is missing.');
            }
            const streams = yield this.getStreams();
            const stream = streams.find(s => s.stream === name);
            if (!stream) {
                return Promise.resolve(undefined);
            }
            return Promise.resolve(stream);
        });
    }
    getStreamField(streamName, fieldName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(streamName) || this.isNullOrWhitespace(fieldName)) {
                throw new Error('Required parameter is missing.');
            }
            const streams = yield this.getStreams();
            const stream = streams.find(stream => stream.stream === streamName);
            if (stream) {
                var field = stream.fields.find(field => field.name === fieldName);
                if (field) {
                    return Promise.resolve(field);
                }
            }
            return Promise.resolve(undefined);
        });
    }
    getTableSchema(tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(tableName)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/schema/${tableName}`;
            const data = yield this.doGet(url);
            if (Object.keys(data).length == 0) {
                return Promise.resolve(undefined);
            }
            return Promise.resolve(data);
        });
    }
    getTableSchemaFieldInfo(tableName, fieldName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(tableName) || this.isNullOrWhitespace(fieldName)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/schema/${tableName}/fieldinfo?fields=${fieldName}`;
            const data = yield this.doGet(url)
                .catch(err => {
                if (err.response.status === 400) {
                    return;
                }
                if (err.response.status === 500) {
                    return;
                }
                throw err;
            });
            if (!data) {
                return Promise.resolve(undefined);
            }
            if (!util_1.isArray(data.fields) || data.fields.length != 1) {
                return Promise.reject('The fields property is not an array or the array does not have 1 member.');
            }
            return Promise.resolve(data.fields[0]);
        });
    }
    getEntity(tableName, fieldName, fieldValue, wait = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(tableName) || this.isNullOrWhitespace(fieldName) || !fieldValue) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/entity/${tableName}/${fieldName}/${encodeURIComponent(fieldValue)}?wait=${wait}`;
            const data = yield this.doGet(url)
                .catch(err => {
                if (err.response.status == 500) {
                    //table not found
                    return Promise.resolve(undefined);
                }
            });
            return Promise.resolve(data);
        });
    }
    getQueries() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/query`;
            return this.doGet(url);
        });
    }
    getQuery(alias) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(alias)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/query/${alias}`;
            const data = yield this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return;
                }
                throw err;
            });
            return Promise.resolve(data);
        });
    }
    upsertQuery(lql) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(lql)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/query`;
            return this.doPost(url, lql);
        });
    }
    deleteQuery(alias) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(alias)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/query/${alias}`;
            const config = {
                url: url,
                method: 'delete',
                headers: this.headers
            };
            const response = yield axios_1.default.request(config);
            if (response.status === 204) {
                return Promise.resolve(true);
            }
            return Promise.resolve(false);
        });
    }
    validateQuery(lql) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(lql)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/query/_validate`;
            const result = new types_1.QueryValidationResult();
            const response = yield this.doPost(url, lql)
                .catch(err => {
                result.success = false;
                result.message = err.response.data.message;
            });
            if (result.success) {
                result.query = response[0];
            }
            return Promise.resolve(result);
        });
    }
    getQueriesGroupedByTable() {
        return __awaiter(this, void 0, void 0, function* () {
            const queries = yield this.getQueries();
            let map = new Map();
            for (let i = 0; i < queries.length; i++) {
                let query = queries[i];
                const table = query.table;
                let values = map.get(table);
                if (!values) {
                    values = [];
                    map.set(table, values);
                }
                values.push(query);
            }
            return Promise.resolve(map);
        });
    }
    toLql(csvText) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(csvText)) {
                throw new Error('Required parameter is missing.');
            }
            const headers = this.headers;
            headers['Content-Type'] = 'application/csv';
            const url = `${this.base_url}/api/query/_tolql`;
            const config = {
                url: url,
                method: 'post',
                data: csvText,
                headers: this.headers
            };
            const response = yield axios_1.default.request(config);
            return Promise.resolve(response.data);
        });
    }
    testQuery(lql, record) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(lql) || !record) {
                throw new Error('Required parameter is missing.');
            }
            const params = qs.stringify(record);
            const url = `${this.base_url}/api/query/_test?${params}`;
            return this.doPost(url, lql);
        });
    }
    collect(stream, data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(stream) || !data) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/collect/json/${stream}`;
            const result = yield this.doPost(url, data);
            return Promise.resolve(result);
        });
    }
    getSegments() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/segment`;
            const segments = yield this.doGet(url);
            if (!segments) {
                return Promise.resolve([]);
            }
            return Promise.resolve(segments);
        });
    }
    getSegment(idOrSlug) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(idOrSlug)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/segment/${idOrSlug}`;
            const segment = yield this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return;
                }
                throw err;
            });
            return Promise.resolve(segment);
        });
    }
    getSegmentGrouping(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const col = new types_1.SegmentGrouping();
            const url = `${this.base_url}/api/segment`;
            const segments = yield this.doGet(url);
            if (!segments) {
                return Promise.resolve(col);
            }
            for (let i = 0; i < segments.length; i++) {
                const segment = segments[i];
                if (ids && ids.indexOf(segment.id) === -1) {
                    continue;
                }
                let target = col.unidentified;
                switch (segment.kind) {
                    case 'segment':
                        target = col.audiences;
                        break;
                    case 'aspect':
                        target = col.characteristics;
                        break;
                }
                if (target) {
                    target.push(segment);
                }
            }
            return Promise.resolve(col);
        });
    }
    getSegmentCollections() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/segmentcollection`;
            const collections = yield this.doGet(url);
            if (!collections) {
                return Promise.resolve([]);
            }
            return Promise.resolve(collections);
        });
    }
    getSegmentCollection(idOrSlug) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(idOrSlug)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/segmentcollection/${idOrSlug}`;
            const collection = yield this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return;
                }
                throw err;
            });
            return Promise.resolve(collection);
        });
    }
    isNullOrWhitespace(value) {
        return (!value || value.trim().length == 0);
    }
    getCampaigns() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/program/campaign`;
            const campaigns = yield this.doGet(url);
            if (!campaigns) {
                return Promise.resolve([]);
            }
            return Promise.resolve(campaigns);
        });
    }
    getCampaign(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(id)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/program/campaign/${id}`;
            const campaign = yield this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(undefined);
                }
                throw err;
            });
            return Promise.resolve(campaign);
        });
    }
    getCampaignVariationsAll() {
        return __awaiter(this, void 0, void 0, function* () {
            const map = new Map();
            const url = `${this.base_url}/api/program/campaign/variation`;
            const variations = yield this.doGet(url);
            if (!variations) {
                return map;
            }
            for (let i = 0; i < variations.length; i++) {
                const variation = variations[i];
                if (!variation) {
                    throw new Error(`Campaign variation at position ${i} is null.`);
                }
                if (!map.has(variation.campaign_id)) {
                    map.set(variation.campaign_id, []);
                }
                const cvars = map.get(variation.campaign_id);
                cvars.push(variation);
            }
            return Promise.resolve(map);
        });
    }
    getCampaignVariations(campaignId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(campaignId)) {
                throw new Error('Required parameter is missing.');
            }
            var map = yield this.getCampaignVariationsAll();
            var variations = map.has(campaignId) ? map.get(campaignId) : [];
            return Promise.resolve(variations);
        });
    }
    getCampaignVariation(variationId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(variationId)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/program/campaign/variation/${variationId}`;
            const variation = yield this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(undefined);
                }
                throw err;
            });
            return Promise.resolve(variation);
        });
    }
    updateCampaignVariation(variationId, variation) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(variationId) || !variation) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/program/campaign/variation/${variationId}`;
            return this.doPost(url, variation);
        });
    }
    getCampaignVariationDetailOverride(variationId) {
        return __awaiter(this, void 0, void 0, function* () {
            var variation = yield this.getCampaignVariation(variationId);
            if (!variation) {
                return Promise.resolve(undefined);
            }
            return Promise.resolve(variation.detail_override);
        });
    }
    updateCampaignVariationDetailOverride(variationId, override) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(variationId) || !override) {
                throw new Error('Required parameter is missing.');
            }
            var variation = new types_1.CampaignVariation();
            variation.detail_override = override;
            return this.updateCampaignVariation(variationId, variation);
        });
    }
    classifyUsingText(text, draft = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/content/doc/classify?draft=${draft}`;
            return this.doPost(url, { "text": text });
        });
    }
    classifyUsingUrl(url, draft = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const url2 = `${this.base_url}/api/content/doc/classify?draft=${draft}`;
            return this.doPost(url2, { "url": url });
        });
    }
    testFunction(functionName, args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(functionName)) {
                throw new Error('Required parameter is missing.');
            }
            const args2 = args ? args.map(arg => `"${arg}"`) : [''];
            const expr = `${functionName}(${args2.join(',')})`;
            const lql = `SELECT ${expr} AS value, email(email) AS email FROM test_stream INTO user BY email ALIAS test_query`;
            var response = yield this.testQuery(lql, { email: "test@test.com" });
            return Promise.resolve(response.value);
        });
    }
    getWhitelistFields(aid) {
        return __awaiter(this, void 0, void 0, function* () {
            if (aid === 0) {
                throw new Error('Required parameter is missing.');
            }
            const account = yield this.getAccount(aid);
            if (!account) {
                throw new Error(`Account ${aid} does not exist or cannot be accessed.`);
            }
            if (!account.whitelist_fields) {
                return Promise.resolve([]);
            }
            if (account.whitelist_fields.length == 1 && this.isNullOrWhitespace(account.whitelist_fields[0])) {
                return Promise.resolve([]);
            }
            return Promise.resolve(account.whitelist_fields);
        });
    }
    /**
     *
     * @param aid
     * @param fieldName
     * @param add
     * @returns true if a change was made; false if no change was made. For example, if the field "email" is already whitelisted and try to add "email" to the whitelist, this function returns false.
     */
    setWhitelistFieldStatus(aid, fieldName, add) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(fieldName)) {
                throw new Error('Required parameter is missing.');
            }
            const fields = yield this.getWhitelistFields(aid);
            const position = fields.indexOf(fieldName);
            if (position == -1) {
                if (!add) {
                    return Promise.resolve(false);
                }
                fields.push(fieldName);
            }
            else {
                if (add) {
                    return Promise.resolve(false);
                }
                fields.splice(position, 1);
                if (fields.length == 0) {
                    //
                    // Lytics will not write an empty array, so
                    // an empty string is used in order to force
                    // the existing value to be replaced.
                    fields.push('');
                }
            }
            const data = {
                "whitelist_fields": fields
            };
            const url = `${this.base_url}/api/account/${aid}`;
            return this.doPost(url, data);
        });
    }
    getTopics(limit = 500) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/content/topic?limit=${limit}`;
            const topics = yield this.doGet(url);
            if (!topics) {
                return Promise.resolve([]);
            }
            return Promise.resolve(topics);
        });
    }
    getTopic(label) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(label)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/content/topic/${label}`;
            const topic = yield this.doGet(url);
            return Promise.resolve(topic);
        });
    }
    getTopicUrls(label, limit = 10) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(label)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/content/topic/${label}/urls?limit=${limit}`;
            const collection = yield this.doGet(url);
            return Promise.resolve(collection);
        });
    }
    getSubscriptions() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/subscription`;
            const subscriptions = yield this.doGet(url);
            if (!subscriptions) {
                return Promise.resolve([]);
            }
            return Promise.resolve(subscriptions);
        });
    }
    getSubscription(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(id)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/subscription/${id}`;
            const subscription = yield this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(undefined);
                }
                throw err;
            });
            return Promise.resolve(subscription);
        });
    }
    createWebhook(config) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!types_1.WebhookConfig.isValid(config)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/subscription`;
            const subscription = yield this.doPost(url, config);
            return Promise.resolve(subscription);
        });
    }
    updateWebhook(id, config) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!id || id.trim().length === 0) {
                throw new Error('Subscription id is required.');
            }
            if (!types_1.WebhookConfig.isValid(config)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/subscription/${id}`;
            const subscription = yield this.doPost(url, config);
            return Promise.resolve(subscription);
        });
    }
    deleteSubscription(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(id)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/subscription/${id}`;
            const subscription = yield this.doDelete(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(false);
                }
                throw err;
            });
            return Promise.resolve(true);
        });
    }
    getTokenScopes() {
        return __awaiter(this, void 0, void 0, function* () {
            return Promise.resolve(types_1.TokenScope.supportedScopes);
        });
    }
    createAccessToken(config) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!config || !config.name || config.name.trim().length === 0) {
                throw new Error('Required parameter is missing.');
            }
            if (!config || !config.scopes || config.scopes.length === 0) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/auth/createtoken`;
            return this.doPost(url, config);
        });
    }
    getAccessTokens(aid) {
        return __awaiter(this, void 0, void 0, function* () {
            const account = yield this.getAccount(aid);
            if (!account) {
                return Promise.resolve([]);
            }
            const url = `${this.base_url}/api/auth?auth_type=api_token&account_id=${account.id}`;
            const tokens = yield this.doGet(url);
            if (!tokens) {
                return Promise.resolve([]);
            }
            return Promise.resolve(tokens);
        });
    }
    getAccessToken(authid, aid) {
        return __awaiter(this, void 0, void 0, function* () {
            const account = yield this.getAccount(aid);
            if (!account) {
                return Promise.resolve(undefined);
            }
            const url = `${this.base_url}/api/auth/${authid}?account_id=${account.id}`;
            return this.doGet(url)
                .catch(err => {
                if (err.response.status == 404) {
                    return Promise.resolve(undefined);
                }
            });
        });
    }
    deleteAccessToken(authid, aid) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(authid)) {
                throw new Error('Required parameter is missing.');
            }
            const account = yield this.getAccount(aid);
            if (!account) {
                return Promise.resolve(false);
            }
            const url = `${this.base_url}/api/auth/${authid}?account_id=${account.id}`;
            const config = {
                url: url,
                method: 'delete',
                headers: this.headers
            };
            const response = yield axios_1.default.request(config);
            if (response.status === 204) {
                return Promise.resolve(true);
            }
            return Promise.resolve(false);
        });
    }
    getAccountSettings() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/account/setting`;
            const settings = yield this.doGet(url);
            return Promise.resolve(settings);
        });
    }
    getAccountSettingsGroupedByCategory() {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = yield this.getAccountSettings();
            let map = new Map();
            for (let i = 0; i < settings.length; i++) {
                let setting = settings[i];
                const category = setting.category;
                let values = map.get(category);
                if (!values) {
                    values = [];
                    map.set(category, values);
                }
                values.push(setting);
            }
            return Promise.resolve(map);
        });
    }
    getAccountSetting(slug) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(slug)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/account/setting/${slug}`;
            return this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(undefined);
                }
                throw err;
            });
        });
    }
    updateAccountSetting(slug, value) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(slug) || value === undefined || value === null) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/account/setting/${slug}`;
            return this.doPut(url, JSON.stringify(value))
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(undefined);
                }
                throw err;
            });
        });
    }
    deleteAccountSetting(slug) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(slug)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/account/setting/${slug}`;
            const subscription = yield this.doDelete(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(false);
                }
                throw err;
            });
            return Promise.resolve(true);
        });
    }
    getDocumentTopics(docUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(docUrl)) {
                throw new Error('Required parameter is missing.');
            }
            //
            //remove protocol from url
            docUrl = docUrl.replace(/(^\w+:|^)\/\//, '');
            const url = `${this.base_url}/api/content/doc?urls=${docUrl}`;
            const result = yield this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(undefined);
                }
                throw err;
            });
            if (util_1.isUndefined(result) || result.total === 0) {
                return Promise.resolve(undefined);
            }
            return Promise.resolve(result.urls[0]);
        });
    }
    uploadData(config, data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!config || this.isNullOrWhitespace(config.stream) || data === undefined || data === null) {
                throw new Error('Required parameter is missing');
            }
            const url = `${this.base_url}/collect/json/${config.stream}?timestamp_field=${config.timestamp_field}&dryrun=${config.dryrun}`;
            return this.doPost(url, data);
        });
    }
    getFragments(table, key, value) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(table) || this.isNullOrWhitespace(key) || this.isNullOrWhitespace(value)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/entity/${table}/${key}/${value}?explain=true`;
            let fragments = yield this.doGet(url);
            if (!fragments) {
                fragments = undefined;
            }
            return Promise.resolve(fragments);
        });
    }
    getSegmentMLModels() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${this.base_url}/api/segmentml`;
            const models = yield this.doGet(url);
            return Promise.resolve(models);
        });
    }
    getSegmentMLModel(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(id)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/segmentml/${id}`;
            return this.doGet(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(undefined);
                }
                else if (err.response.status === 500 && err.response.message === 'Could not fetch segment model') {
                    return Promise.resolve(undefined);
                }
                throw err;
            });
        });
    }
    createSegmentMLModel(config) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!config || this.isNullOrWhitespace(config.source) || this.isNullOrWhitespace(config.target)) {
                throw new Error('Required parameter is missing.');
            }
            const url = `${this.base_url}/api/segmentml`;
            const model = yield this.doPost(url, config);
            return Promise.resolve(model);
        });
    }
    deleteSegmentMLModel(name) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isNullOrWhitespace(name)) {
                throw new Error('Required parameter is missing.');
            }
            name = types_1.SegmentMLModel.getModelNameWithoutGeneration(name);
            const url = `${this.base_url}/api/segmentml/${name}`;
            var result = yield this.doDelete(url)
                .catch(err => {
                if (err.response.status === 404) {
                    return Promise.resolve(false);
                }
                throw err;
            });
            if (util_1.isNullOrUndefined(result)) {
                result = true;
            }
            return Promise.resolve(result);
        });
    }
}
exports.LyticsClient = LyticsClient;
//# sourceMappingURL=LyticsClient.js.map