"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const LyticsClient_1 = require("../LyticsClient");
const chai_1 = require("chai");
const types_1 = require("../types");
const url_1 = require("url");
const util_1 = require("util");
const settings = require('./settings');
const apikey = settings.lytics_apikey;
const aid = settings.lytics_aid;
before(function (done) {
    if (apikey.trim().length == 0 || aid == 0) {
        done(new Error('The variables apikey and aid must be specified before tests can run.'));
        return;
    }
    done();
});
describe('getAccounts', function () {
    it('should return an array of accounts', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var accounts = yield lytics.getAccounts();
            chai_1.assert.isDefined(accounts);
            chai_1.assert.equal(accounts.length, 1);
            chai_1.assert.equal(accounts[0].aid, aid);
        });
    });
});
describe('getAccount', function () {
    it('should return undefined when a mismatched account id is used', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var account = yield lytics.getAccount(123);
            chai_1.assert.isUndefined(account);
        });
    });
    it('should return an account when a matched account id is used', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var account = yield lytics.getAccount(aid);
            chai_1.assert.isDefined(account);
            chai_1.assert.equal(account.aid, aid);
        });
    });
});
describe('getStreams', function () {
    it('should return an array of data streams', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var streams = yield lytics.getStreams();
            chai_1.assert.isDefined(streams);
            chai_1.assert.isTrue(streams.length >= 2);
            chai_1.assert.isDefined(streams.find(s => s.stream == 'default'));
            chai_1.assert.isDefined(streams.find(s => s.stream == 'lytics_content_enrich'));
        });
    });
    it('should return undefined when a data stream that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var stream = yield lytics.getStream('87y789fdfn');
            chai_1.assert.isUndefined(stream);
        });
    });
    it('should return a data stream', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var stream = yield lytics.getStream('default');
            chai_1.assert.isDefined(stream);
            chai_1.assert.equal(stream.stream, 'default');
        });
    });
});
describe('getStreamField', function () {
    it('should return a field from the data stream', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var field = yield lytics.getStreamField('default', '_uid');
            chai_1.assert.isDefined(field);
            chai_1.assert.equal(field.name, '_uid');
        });
    });
    it('should return undefined when a data stream that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var field = yield lytics.getStreamField('3t4w4tsg', '_uid');
            chai_1.assert.isUndefined(field);
        });
    });
    it('should return undefined when a field that does not exist in the data stream is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var field = yield lytics.getStreamField('default', 'dfgdsrgt43');
            chai_1.assert.isUndefined(field);
        });
    });
});
describe('getTableSchema', function () {
    it('should return undefined when a table that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var schema = yield lytics.getTableSchema('xxx');
            chai_1.assert.isUndefined(schema);
        });
    });
    it('should return an object when the user table is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var schema = yield lytics.getTableSchema('user');
            chai_1.assert.isDefined(schema);
            chai_1.assert.equal(schema.name, 'user');
            chai_1.assert.isTrue(schema.columns.length > 0);
            chai_1.assert.isTrue(schema.by_fields.length > 0);
        });
    });
});
describe('getTableSchemaFieldInfo', function () {
    it('should return undefined when a table that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var info = yield lytics.getTableSchemaFieldInfo('xxx', 'yyy');
            chai_1.assert.isUndefined(info);
        });
    });
    it('should return undefined when the table exists but the field does not', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var info = yield lytics.getTableSchemaFieldInfo('user', 'yyy');
            chai_1.assert.isUndefined(info);
        });
    });
    it('should return an object when the user table and email field are specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const info = yield lytics.getTableSchemaFieldInfo('user', '_uid');
            chai_1.assert.isDefined(info);
            chai_1.assert.equal(info.field, '_uid');
            const counts = types_1.TableSchemaFieldInfo.getTermCounts(info);
            chai_1.assert.isNotEmpty(counts);
        });
    });
});
describe('getTableSchemaFieldInfo', function () {
    it('should return undefined when a table that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var info = yield lytics.getTableSchemaFieldInfo('xxx', 'yyy');
            chai_1.assert.isUndefined(info);
        });
    });
    it('should return undefined when the table exists but the field does not', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var info = yield lytics.getTableSchemaFieldInfo('user', 'yyy');
            chai_1.assert.isUndefined(info);
        });
    });
    it('should return a field info object when the user table is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const info = yield lytics.getTableSchemaFieldInfo('user', '_uid');
            chai_1.assert.isDefined(info);
            chai_1.assert.equal(info.field, '_uid');
            const counts = types_1.TableSchemaFieldInfo.getTermCounts(info);
            chai_1.assert.isNotEmpty(counts);
        });
    });
});
describe('getEntity', function () {
    it('should return undefined when a table that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var entity = yield lytics.getEntity('xxx', 'yyy', 'zzz');
            chai_1.assert.isUndefined(entity);
        });
    });
    it('should return undefined when the table exists but the field does not', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var entity = yield lytics.getEntity('user', 'yyy', 'zzz');
            chai_1.assert.isUndefined(entity);
        });
    });
    it('should return undefined when the table and field exist but the value does not match an entity', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var entity = yield lytics.getEntity('user', 'email', 'zzz');
            chai_1.assert.isUndefined(entity);
        });
    });
    it('should return an object when the table, field and value match an existing entity', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var entity = yield lytics.getEntity('user', '_uid', '114940.99527438209');
            chai_1.assert.isDefined(entity);
            chai_1.assert.equal(entity._uid, '114940.99527438209');
        });
    });
    it('should return an object when wait is true', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var entity = yield lytics.getEntity('user', '_uid', '114940.99527438209', true);
            chai_1.assert.isDefined(entity);
            chai_1.assert.equal(entity._uid, '114940.99527438209');
        });
    });
    it('should return an object when wait is false', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var entity = yield lytics.getEntity('user', '_uid', '114940.99527438209', false);
            chai_1.assert.isDefined(entity);
            chai_1.assert.equal(entity._uid, '114940.99527438209');
        });
    });
    //
    it('should return an object when a value that must be url-encoded is used', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var entity = yield lytics.getEntity('user', 'email', 'adam.conn+test10@gmail.com', false);
            chai_1.assert.isDefined(entity);
            chai_1.assert.equal(entity.email, 'adam.conn+test10@gmail.com');
        });
    });
});
describe('getQueries', function () {
    it('should return an array of queries', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var queries = yield lytics.getQueries();
            chai_1.assert.isDefined(queries);
            chai_1.assert.isNotEmpty(queries);
        });
    });
});
describe('getQuery', function () {
    it('should return undefined when a query that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var query = yield lytics.getQuery('xxx');
            chai_1.assert.isUndefined(query);
        });
    });
    it('should return a query when a query that exists is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var query = yield lytics.getQuery('user_web_default');
            chai_1.assert.isDefined(query);
        });
    });
});
describe('upsertQuery', function () {
    it('should throw an error when invalid LQL is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var wasThrown = false;
            try {
                yield lytics.upsertQuery('xxx');
            }
            catch (err) {
                wasThrown = true;
            }
            chai_1.assert.isTrue(wasThrown);
        });
    });
    it('should return when valid LQL is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const alias = '_DELETE_me_unit_test'.toLowerCase();
            const lql = `SELECT email FROM test_stream INTO user BY email ALIAS ${alias}`;
            let query = yield lytics.getQuery(alias);
            chai_1.assert.isUndefined(query, 'The test query already exists. The test cannot continue.');
            var query2 = yield lytics.upsertQuery(lql);
            chai_1.assert.isDefined(query2);
            chai_1.assert.isNotEmpty(query2);
            chai_1.assert.equal(query2[0].alias, alias);
            query = yield lytics.getQuery(alias);
            chai_1.assert.isDefined(query);
            chai_1.assert.equal(query.alias, alias);
            chai_1.assert.equal(query.text, lql);
            const wasDeleted = yield lytics.deleteQuery(alias);
            chai_1.assert.isTrue(wasDeleted);
        });
    });
});
describe('validateQuery', function () {
    it('validation fails with invalid LQL', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const lql = 'SELECT email(`email) AS email FROM test INTO user BY email ALIAS test';
            let result = yield lytics.validateQuery(lql);
            chai_1.assert.isFalse(result.success);
        });
    });
    it('validation is successful with valid LQL', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const lql = 'SELECT email(`email`) AS email FROM test INTO user BY email ALIAS test';
            let result = yield lytics.validateQuery(lql);
            chai_1.assert.isTrue(result.success);
        });
    });
});
describe('getQueriesGroupedByTable', function () {
    it('gets a map of the queries', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var queries = yield lytics.getQueriesGroupedByTable();
            chai_1.assert.isDefined(queries);
            chai_1.assert.isNotEmpty(queries.get('user'));
        });
    });
});
describe('toLql', function () {
    it('generates LQL when text is provided', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const csv = `user_id,event,ts
        7456,login,"2012-10-17T17:29:39.738Z"
        1234,"a quoted bad event","2012-12-11T19:53:31.547Z"`;
            const response = yield lytics.toLql(csv);
            chai_1.assert.isDefined(response);
        });
    });
});
describe('testQuery', function () {
    it('evaluates properly when valid LQL and data are provided', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const lql = `
        SELECT

        first                   AS first_name               SHORTDESC "First Name"
        , last                  AS last_name                SHORTDESC "Last Name"
        -- By Fields/Unique Keys
        , email(email)          AS email                    SHORTDESC "Email"
    
    FROM test_stream
    INTO user BY email 
    ALIAS test_query
            `;
            const data = { first: "Adam", last: "Conn", email: "adam.conn@lytics.com" };
            const response = yield lytics.testQuery(lql, data);
            chai_1.assert.isDefined(response);
            chai_1.assert.equal(response.first_name, "Adam");
            chai_1.assert.equal(response.last_name, "Conn");
            chai_1.assert.equal(response.email, "adam.conn@lytics.com");
        });
    });
});
describe('collect', function () {
    it('should not throw an error when data is written to a stream', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const result = yield lytics.collect('test-stream-unit-test', { name: 'aaa' });
            chai_1.assert.isDefined(result);
            chai_1.assert.equal(result.message_count, 1);
        });
    });
});
describe('getSegmentGroup', function () {
    it('should get all segments when no parameter is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const segments = yield lytics.getSegmentGrouping();
            chai_1.assert.isDefined(segments);
            chai_1.assert.isDefined(segments.audiences);
            chai_1.assert.isDefined(segments.characteristics);
            chai_1.assert.isDefined(segments.unidentified);
        });
    });
    it('should get 1 segment when only 1 segment is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const segments = yield lytics.getSegmentGrouping(['fcfb11e24b93964cba3aa2527ab913a3']);
            chai_1.assert.isDefined(segments);
            chai_1.assert.isDefined(segments.audiences);
            chai_1.assert.equal(segments.audiences.length, 1);
            chai_1.assert.isDefined(segments.characteristics);
            chai_1.assert.equal(segments.characteristics.length, 0);
            chai_1.assert.isDefined(segments.unidentified);
            chai_1.assert.equal(segments.unidentified.length, 0);
        });
    });
    it('should get 0 segment when a segment is specified that does not exist', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const segments = yield lytics.getSegmentGrouping(['aaaaa']);
            chai_1.assert.isDefined(segments);
            chai_1.assert.isDefined(segments.audiences);
            chai_1.assert.equal(segments.audiences.length, 0);
            chai_1.assert.isDefined(segments.characteristics);
            chai_1.assert.equal(segments.characteristics.length, 0);
            chai_1.assert.isDefined(segments.unidentified);
            chai_1.assert.equal(segments.unidentified.length, 0);
        });
    });
});
describe('getSegments', function () {
    it('should get all segments', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const segments = yield lytics.getSegments();
            chai_1.assert.isDefined(segments);
            chai_1.assert.isTrue(segments.length > 0);
        });
    });
});
describe('getSegment', function () {
    it('should get undefined is the value specified does not match an existing segment', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const segment = yield lytics.getSegment('asdasd');
            chai_1.assert.isUndefined(segment);
        });
    });
    it('should get the segment by slug', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const segment = yield lytics.getSegment('all');
            chai_1.assert.isDefined(segment);
            chai_1.assert.equal(segment.slug_name, 'all');
        });
    });
    it('should get the segment by id', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const segment = yield lytics.getSegment('fcfb11e24b93964cba3aa2527ab913a3');
            chai_1.assert.isDefined(segment);
            chai_1.assert.equal(segment.id, 'fcfb11e24b93964cba3aa2527ab913a3');
        });
    });
});
describe('getSegmentCollections', function () {
    it('should get all segment collections', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const collections = yield lytics.getSegmentCollections();
            chai_1.assert.isDefined(collections);
            chai_1.assert.isTrue(collections.length > 0);
        });
    });
});
describe('getSegmentCollection', function () {
    it('should get undefined is the value specified does not match an existing segment collection', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const collection = yield lytics.getSegmentCollection('asdasd');
            chai_1.assert.isUndefined(collection);
        });
    });
    it('should get the segment collection by slug', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const collection = yield lytics.getSegmentCollection('segmentation_email_capture');
            chai_1.assert.isDefined(collection);
            chai_1.assert.equal(collection.slug_name, 'segmentation_email_capture');
        });
    });
    it('should get the segment collection by id', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const collection = yield lytics.getSegmentCollection('00d8698ed83dc7edf6e9e31c77a60e74');
            chai_1.assert.isDefined(collection);
            chai_1.assert.equal(collection.id, '00d8698ed83dc7edf6e9e31c77a60e74');
        });
    });
});
describe('getCampaigns', function () {
    it('should return campaigns', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var campaigns = yield lytics.getCampaigns();
            chai_1.assert.isDefined(campaigns);
            chai_1.assert.isArray(campaigns);
        });
    });
});
describe('getCampaign', function () {
    it('should return the specified campaign when the campaign exists', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const campaignId = "38adaf20d5cad612b013ae2082cd8ae3";
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var campaign = yield lytics.getCampaign(campaignId);
            chai_1.assert.isDefined(campaign);
            chai_1.assert.equal(campaign.id, campaignId);
        });
    });
    it('should return undefined when the campaign does not exist', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const campaignId = "sdfasdfasdfasfda";
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var campaign = yield lytics.getCampaign(campaignId);
            chai_1.assert.isUndefined(campaign);
        });
    });
});
describe('getCampaignVariations', function () {
    it('should return an array when a campaign that exists is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const campaignId = "38adaf20d5cad612b013ae2082cd8ae3";
            var variations = yield lytics.getCampaignVariations(campaignId);
            chai_1.assert.isDefined(variations);
            chai_1.assert.isArray(variations);
            chai_1.assert.isNotEmpty(variations);
            for (let i = 0; i < variations.length; i++) {
                chai_1.assert.equal(variations[i].campaign_id, campaignId);
            }
        });
    });
    it('should return an empty array when a campaign that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var variations = yield lytics.getCampaignVariations("sdfsdfsdfjkhkjsdf");
            chai_1.assert.isDefined(variations);
            chai_1.assert.isArray(variations);
            chai_1.assert.isEmpty(variations);
        });
    });
});
describe('getCampaignVariation', function () {
    it('should return an object when a campaign variation that exists is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const variationId = "c7bcd903694c66d1d72371b51286f7da";
            var variation = yield lytics.getCampaignVariation(variationId);
            chai_1.assert.isDefined(variation);
            chai_1.assert.equal(variation.id, variationId);
        });
    });
    it('should return undefined when a campaign variation that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var variation = yield lytics.getCampaignVariation("sdfsdfsdfjkhkjsdf");
            chai_1.assert.isUndefined(variation);
        });
    });
});
describe('updateCampaignVariation', function () {
    it('should return the updated variation when a campaign variation that exists is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const variationId = "e95f1ed37d357fff00aa938ca947e407";
            const layout = "slideout";
            const attachment = "bottom-left";
            const variation = yield lytics.getCampaignVariation(variationId);
            chai_1.assert.isDefined(variation);
            variation.detail.layout = layout;
            variation.detail.attachment = attachment;
            const variation2 = yield lytics.updateCampaignVariation(variationId, variation);
            chai_1.assert.isDefined(variation2);
            chai_1.assert.equal(variation2.id, variationId);
            chai_1.assert.equal(variation2.detail.layout, layout);
            chai_1.assert.equal(variation2.detail.attachment, attachment);
        });
    });
});
describe('getCampaignVariationDetailOverride', function () {
    it('should return an object when a campaign variation that exists is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const variationId = "5ad0b92b1eec4903fa4c21a3046d72a8";
            var override = yield lytics.getCampaignVariationDetailOverride(variationId);
            chai_1.assert.isDefined(override);
            chai_1.assert.isFalse(override.fields.email);
        });
    });
    it('should return undefined when a campaign variation that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var override = yield lytics.getCampaignVariation("sdfsdfsdfjkhkjsdf");
            chai_1.assert.isUndefined(override);
        });
    });
});
describe('updateCampaignVariationDetailOverride', function () {
    it('should return the updated variation when a campaign variation that exists is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const variationId = "5ad0b92b1eec4903fa4c21a3046d72a8";
            const override = yield lytics.getCampaignVariationDetailOverride(variationId);
            chai_1.assert.isDefined(override);
            chai_1.assert.isFalse(override.fields.email);
            override.fields.email = true;
            const variation2 = yield lytics.updateCampaignVariationDetailOverride(variationId, override);
            chai_1.assert.isDefined(variation2);
            const override2 = yield lytics.getCampaignVariationDetailOverride(variationId);
            chai_1.assert.isDefined(override2);
            chai_1.assert.isTrue(override2.fields.email);
            override2.fields.email = false;
            const variation3 = yield lytics.updateCampaignVariationDetailOverride(variationId, override2);
            chai_1.assert.isDefined(variation3);
            const override3 = yield lytics.getCampaignVariationDetailOverride(variationId);
            chai_1.assert.isDefined(override3);
            chai_1.assert.isFalse(override3.fields.email);
        });
    });
});
describe('classifyUsingText', function () {
    it('should return an object when enough text is provided', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const text = '50% of the Dawn of Love is an oil painting by English artist William Etty, first exhibited in 1828. Loosely based on a passage from John Miltons 1634 Comus, it shows Venus leaning across to wake the sleeping Love by stroking his wings. It was very poorly received when first exhibited; the stylised Venus was thought unduly influenced by foreign artists such as Rubens as well as being overly voluptuous and unrealistically coloured, while the painting as a whole was considered tasteless and obscene. The Dawn of Love was omitted from the major 1849 retrospective exhibition of Ettys works, and its exhibition in Glasgow in 1899 drew complaints for its supposed obscenity. In 1889 it was bought by Merton Russell-Cotes, and has remained in the collection of the Russell-Cotes Art Gallery & Museum ever since.';
            var classification = yield lytics.classifyUsingText(text);
            chai_1.assert.isDefined(classification);
        });
    });
    it('throws an error when not enough text is provided', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const text = 'This is a test.';
            try {
                var classification = yield lytics.classifyUsingText(text);
            }
            catch (err) {
                return Promise.resolve();
            }
            return Promise.reject('No error was thrown.');
        });
    });
});
describe('classifyUsingUrl', function () {
    it('should return an object when enough text is provided', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const url = "https://www.google.com";
            var classification = yield lytics.classifyUsingUrl(url);
            chai_1.assert.isDefined(classification);
        });
    });
    it('throws an error when an invalid url is provided', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const url = 'not a url';
            try {
                var classification = yield lytics.classifyUsingUrl(url);
            }
            catch (err) {
                return Promise.resolve();
            }
            return Promise.reject('No error was thrown.');
        });
    });
});
describe('testFunction', function () {
    it('The tobool function returns true.', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var result = yield lytics.testFunction('tobool', ['true']);
            chai_1.assert.isTrue(result);
        });
    });
    it('The toint function returns an integer.', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var result = yield lytics.testFunction('toint', ['100']);
            chai_1.assert.equal(result, 100);
        });
    });
    it('The qs function returns a string.', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var result = yield lytics.testFunction('qs', ['https://www.lytics.com/test?name=First+Last', 'name']);
            chai_1.assert.equal(result, 'First Last');
        });
    });
});
describe('setWhitelistFieldStatus', function () {
    it('Add an already existing field returns false.', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const before = yield lytics.getWhitelistFields(aid);
            const resetWhenFinished = before.indexOf('email') == -1;
            if (resetWhenFinished) {
                yield lytics.setWhitelistFieldStatus(aid, 'email', true);
            }
            var result = yield lytics.setWhitelistFieldStatus(aid, 'email', true);
            chai_1.assert.isFalse(result);
            if (resetWhenFinished) {
                result = yield lytics.setWhitelistFieldStatus(aid, 'email', false);
                chai_1.assert.isTrue(result);
            }
            const after = yield lytics.getWhitelistFields(aid);
            before.sort();
            after.sort();
            chai_1.assert.equal(after.length, before.length);
            for (var i = 0; i < before.length; i++) {
                chai_1.assert.equal(after[i], before[i]);
            }
        });
    });
});
describe('getTopics', function () {
    it('should get all topics with no limit set', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const topics = yield lytics.getTopics();
            chai_1.assert.isDefined(topics);
            chai_1.assert.isTrue(topics.length > 0);
            chai_1.assert.isTrue(topics.length <= 500);
        });
    });
    it('should get 2 topics when limit is set to 2', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const topics = yield lytics.getTopics(2);
            chai_1.assert.isDefined(topics);
            chai_1.assert.equal(topics.length, 2);
        });
    });
});
describe('getTopic', function () {
    it('should get undefined when the specified label does not match an existing topic', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const topic = yield lytics.getTopic('asdasd');
            chai_1.assert.isDefined(topic);
            chai_1.assert.equal(topic.doc_count, 0);
        });
    });
    it('should get the topic that matches the specified label', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const topic = yield lytics.getTopic('Protein');
            chai_1.assert.isDefined(topic);
            chai_1.assert.equal(topic.label, 'Protein');
            chai_1.assert.isTrue(topic.doc_count > 0);
        });
    });
});
describe('getTopicUrls', function () {
    it('should get an empty collection when no topic matches the specified label', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const urls = yield lytics.getTopicUrls('asdasd');
            chai_1.assert.isDefined(urls);
            chai_1.assert.equal(urls.total, 0);
            chai_1.assert.isDefined(urls.urls);
            chai_1.assert.equal(urls.urls.length, 0);
        });
    });
    it('should get a populated collection when a topic matches the specified label', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const urls = yield lytics.getTopicUrls('Protein');
            chai_1.assert.isDefined(urls);
            chai_1.assert.isTrue(urls.total > 0);
            chai_1.assert.isDefined(urls.urls);
            chai_1.assert.isTrue(urls.urls.length > 0);
        });
    });
});
describe('getSubscriptions', function () {
    it('should return subscriptions', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var subscriptions = yield lytics.getSubscriptions();
            chai_1.assert.isDefined(subscriptions);
            chai_1.assert.isArray(subscriptions);
        });
    });
});
describe('getSubscription', function () {
    it('should return the specified subscription when the subscription exists', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const subscriptionId = "17e5c8c1a9f8e467d6786497a32c9ffc";
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var subscription = yield lytics.getSubscription(subscriptionId);
            chai_1.assert.isDefined(subscription);
            chai_1.assert.equal(subscription.id, subscriptionId);
        });
    });
    it('should return undefined when the subscription does not exist', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const subscriptionId = "sdfasdfasdfasfda";
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            var subscription = yield lytics.getSubscription(subscriptionId);
            chai_1.assert.isUndefined(subscription);
        });
    });
});
describe('createWebhook', function () {
    it('should throw error when values are missing from the webhook config object', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            lytics.createWebhook(new types_1.WebhookConfig())
                .then(result => chai_1.assert.isTrue(false))
                .catch(err => chai_1.assert.isTrue(true));
        });
    });
    it('should return a subscription when the webhook config object is properly configured', function () {
        const lytics = new LyticsClient_1.LyticsClient(apikey);
        const config = new types_1.WebhookConfig();
        const d = new Date();
        config.name = `CREATE UNIT TEST ${d}`;
        config.description = `subscription for unit test ${d}`;
        config.segment_ids.push('c5cbd2543e9c1fb33b78bcc5b58dec3a'); //all
        config.segment_ids.push('809ab71ce55ec096a9b4bdff0c108456'); //new
        config.webhook_url = new url_1.URL('https://localhost/test');
        lytics.createWebhook(config).then(subscription => {
            chai_1.assert.isDefined(subscription);
            chai_1.assert.equal(subscription.name, config.name);
            chai_1.assert.equal(subscription.description, config.description);
            chai_1.assert.equal(subscription.segment_ids.length, config.segment_ids.length);
            //
            //delete subscription
            const id = subscription.id;
            lytics.deleteSubscription(id).then(wasDeleted => {
                chai_1.assert.isTrue(wasDeleted);
                //
                //ensure subscription does not exist
                lytics.getSubscription(id).then(subscription => {
                    chai_1.assert.isUndefined(subscription);
                });
            });
        });
    });
});
describe('updateWebhook', function () {
    it('should throw error when no id is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            lytics.updateWebhook('', new types_1.WebhookConfig())
                .then(result => chai_1.assert.isTrue(false))
                .catch(err => chai_1.assert.isTrue(true));
        });
    });
    it('should throw error when values are missing from the webhook config object', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            lytics.updateWebhook('aaaa', new types_1.WebhookConfig())
                .then(result => chai_1.assert.isTrue(false))
                .catch(err => chai_1.assert.isTrue(true));
        });
    });
    it('should return a subscription when the webhook config object is properly configured', function () {
        const lytics = new LyticsClient_1.LyticsClient(apikey);
        const config = new types_1.WebhookConfig();
        const d = new Date();
        config.name = `UPDATE UNIT TEST ${d}`;
        config.description = `subscription for unit test ${d}`;
        config.segment_ids.push('c5cbd2543e9c1fb33b78bcc5b58dec3a'); //all
        config.segment_ids.push('809ab71ce55ec096a9b4bdff0c108456'); //new
        config.webhook_url = new url_1.URL('https://localhost/test');
        lytics.createWebhook(config).then(subscription => {
            chai_1.assert.isDefined(subscription);
            chai_1.assert.equal(subscription.name, config.name);
            chai_1.assert.equal(subscription.description, config.description);
            chai_1.assert.equal(subscription.segment_ids.length, config.segment_ids.length);
            config.name = `xxx ${config.name}`;
            lytics.updateWebhook(subscription.id, config).then(subscription => {
                chai_1.assert.isDefined(subscription);
                chai_1.assert.equal(subscription.name, config.name);
                //
                //delete subscription
                const id = subscription.id;
                lytics.deleteSubscription(id).then(wasDeleted => {
                    chai_1.assert.isTrue(wasDeleted);
                    //
                    //ensure subscription does not exist
                    lytics.getSubscription(id).then(subscription => {
                        chai_1.assert.isUndefined(subscription);
                    });
                });
            });
        });
    });
});
describe('deleteSubscription', function () {
    it('should throw error when trying to delete a subscription that does not exist', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            lytics.deleteSubscription('asdasdasda')
                .then(result => chai_1.assert.isTrue(false))
                .catch(err => chai_1.assert.isTrue(true));
        });
    });
    it('should return when a subscription is deleted', function () {
        //
        //create a subscription
        const lytics = new LyticsClient_1.LyticsClient(apikey);
        const config = new types_1.WebhookConfig();
        const d = new Date();
        config.name = `DELETE UNIT TEST ${d}`;
        config.description = `subscription for unit test ${d}`;
        config.segment_ids.push('c5cbd2543e9c1fb33b78bcc5b58dec3a'); //all
        config.segment_ids.push('809ab71ce55ec096a9b4bdff0c108456'); //new
        config.webhook_url = new url_1.URL('https://localhost/test');
        lytics.createWebhook(config)
            .then(subscription => {
            chai_1.assert.isDefined(subscription);
            chai_1.assert.isDefined(subscription.id);
            const id = subscription.id;
            //
            //ensure subscription exists
            lytics.getSubscription(subscription.id).then(subscription => {
                chai_1.assert.equal(subscription.id, id);
                //
                //delete subscription
                lytics.deleteSubscription(id).then(wasDeleted => {
                    chai_1.assert.isTrue(wasDeleted);
                    //
                    //ensure subscription does not exist
                    lytics.getSubscription(id).then(subscription => {
                        chai_1.assert.isUndefined(subscription);
                    });
                });
            });
        });
    });
});
describe('createAccessToken', function () {
    it('should throw error when trying no values are set on the config object', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const config = new types_1.CreateAccessTokenConfig();
            lytics.createAccessToken(config)
                .then(result => chai_1.assert.isTrue(false))
                .catch(err => chai_1.assert.isTrue(true));
        });
    });
    it('should return an access token', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const config = new types_1.CreateAccessTokenConfig();
            config.expires = '10m';
            config.name = 'unit_test_token';
            config.scopes.push('admin');
            const token = yield lytics.createAccessToken(config);
            chai_1.assert.isDefined(token);
            chai_1.assert.equal(token.description, config.name);
            chai_1.assert.isDefined(token.config);
            const apikey2 = types_1.LyticsAccessTokenReader.getApiKey(token);
            chai_1.assert.isDefined(apikey2);
            const client = new LyticsClient_1.LyticsClient(apikey2);
            const account = yield client.getAccount(aid);
            chai_1.assert.isDefined(account);
        });
    });
});
describe('getAccessTokens', function () {
    it('should return undefined if an aid that does not exist is used', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            lytics.getAccessTokens(111231231)
                .then(result => chai_1.assert.isTrue(false))
                .catch(err => chai_1.assert.isTrue(true));
        });
    });
    it('should return an array', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const tokens = yield lytics.getAccessTokens(aid);
            chai_1.assert.isDefined(tokens);
        });
    });
});
describe('getAccessToken', function () {
    it('should return undefined when an access token that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const token = yield lytics.getAccessToken('xxxxx', aid);
            chai_1.assert.isUndefined(token);
        });
    });
    it('should return an access token', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const config = new types_1.CreateAccessTokenConfig();
            config.name = 'test-token';
            config.expires = '10m';
            config.scopes.push('admin');
            const token = yield lytics.createAccessToken(config);
            chai_1.assert.isDefined(token);
            chai_1.assert.isDefined(token.id);
            chai_1.assert.equal(token.description, config.name);
            const token2 = yield lytics.getAccessToken(token.id, aid);
            chai_1.assert.isDefined(token2);
            chai_1.assert.equal(token2.id, token.id);
            const wasDeleted = yield lytics.deleteAccessToken(token.id, aid);
            chai_1.assert.isTrue(wasDeleted);
            const token3 = yield lytics.getAccessToken(token.id, aid);
            chai_1.assert.isUndefined(token3);
        });
    });
});
describe('getTokenScopes', function () {
    it('should return an array', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const scopes = yield lytics.getTokenScopes();
            chai_1.assert.isDefined(scopes);
            chai_1.assert.isTrue(util_1.isArray(scopes));
        });
    });
});
describe('getAccountSettings', function () {
    it('should return an array of account settings', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const settings = yield lytics.getAccountSettings();
            chai_1.assert.isDefined(settings);
            chai_1.assert.isTrue(util_1.isArray(settings));
        });
    });
});
describe('getAccountSettingsGroupedByCategory', function () {
    it('should return a map of account settings', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const map = yield lytics.getAccountSettingsGroupedByCategory();
            chai_1.assert.isDefined(map);
            for (let key of map.keys()) {
                const settings = map.get(key);
                chai_1.assert.isDefined(settings);
                for (var i = 0; i < settings.length; i++) {
                    chai_1.assert.equal(settings[i].category, key);
                }
            }
        });
    });
});
describe('getAccountSetting', function () {
    it('should return undefined if an invalid slug is used', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const slug = 'xxxx';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const setting = yield lytics.getAccountSetting(slug);
            chai_1.assert.isUndefined(setting);
        });
    });
    it('should return an object when a valid slug is used', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const slug = 'api_whitelist_domains';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const setting = yield lytics.getAccountSetting(slug);
            chai_1.assert.isDefined(setting);
            chai_1.assert.equal(setting.slug, slug);
        });
    });
});
describe('updateAccountSetting', function () {
    it('should return undefined if the slug of a setting that does not exist is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const slug = 'xxxx';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const setting = yield lytics.updateAccountSetting(slug, 'aaaaa');
            chai_1.assert.isUndefined(setting);
        });
    });
    it('should throw an error if no value is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const slug = 'api_whitelist_domains';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            try {
                const setting = yield lytics.updateAccountSetting(slug, undefined);
                chai_1.assert.isTrue(false, 'An error should have been thrown.');
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
            try {
                const setting = yield lytics.updateAccountSetting(slug, null);
                chai_1.assert.isTrue(false, 'An error should have been thrown.');
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
        });
    });
    it('should throw an error if an invalid value is specified', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const slug = 'api_whitelist_domains';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            try {
                const setting = yield lytics.updateAccountSetting(slug, false);
                chai_1.assert.isTrue(false, 'An error should have been thrown if false is set for an array setting.');
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
        });
    });
    it('should return an object when an existing account setting is updated with a compatible value', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const newValue = ['www.lytics.com', 'new.lytics.com'];
            const slug = 'api_whitelist_domains';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const setting = yield lytics.updateAccountSetting(slug, newValue);
            chai_1.assert.isDefined(setting);
            chai_1.assert.equal(setting.slug, slug);
            chai_1.assert.isDefined(setting.value);
            chai_1.assert.isTrue(util_1.isArray(setting.value));
            chai_1.assert.equal(setting.value.length, newValue.length);
            for (let i = 0; i < newValue.length; i++) {
                chai_1.assert.equal(setting.value[i], newValue[i]);
            }
            const result = yield lytics.deleteAccountSetting(slug);
            chai_1.assert.isTrue(result);
        });
    });
});
describe('getDocumentTopics', function () {
    it('should throw an error if an empty url is provided', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            try {
                yield lytics.getDocumentTopics('  ');
                chai_1.assert.isTrue(false);
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
        });
    });
    it('should return undefined if a url that has not been crawled is provided', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const topics = yield lytics.getDocumentTopics('xxxxxxx');
            chai_1.assert.isUndefined(topics);
        });
    });
    it('should return topic if a url that has been crawled is provided', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const url = 'adamconn.lyticsdemo.com/no-cook-yogurt-cups.html';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const topics = yield lytics.getDocumentTopics(url);
            chai_1.assert.isDefined(topics);
            chai_1.assert.equal(topics.url, url);
            const topics2 = yield lytics.getDocumentTopics(`https://${url}`);
            chai_1.assert.isDefined(topics2);
            chai_1.assert.equal(topics2.url, url);
        });
    });
});
describe('uploadData', function () {
    it('should throw an error if parameters are missing', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            try {
                yield lytics.uploadData(new types_1.DataUploadConfig('xxxxx'), undefined);
                chai_1.assert.isTrue(false);
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
        });
    });
    it('should return that one message was handled', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const config = new types_1.DataUploadConfig('xxxxx');
            config.dryrun = true;
            const data = { name: "Test User 1" };
            const result = yield lytics.uploadData(config, data);
            chai_1.assert.isDefined(result);
            chai_1.assert.equal(result.message_count, 1);
            chai_1.assert.equal(result.rejected_count, 0);
        });
    });
    it('should return that multiple message were handled', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const config = new types_1.DataUploadConfig('xxxxx');
            config.dryrun = true;
            const data = [
                { name: "Test User 1" },
                { name: "Test User 2" }
            ];
            const result = yield lytics.uploadData(config, data);
            chai_1.assert.isDefined(result);
            chai_1.assert.equal(result.message_count, data.length);
            chai_1.assert.equal(result.rejected_count, 0);
        });
    });
});
describe('getSegmentMLModels', function () {
    it('should return an array of models', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const models = yield lytics.getSegmentMLModels();
            chai_1.assert.isDefined(models);
            chai_1.assert.isTrue(util_1.isArray(models));
        });
    });
});
describe('getSegmentMLModel', function () {
    it('should return undefined if an invalid id is used', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const id = 'xxxx';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const model = yield lytics.getSegmentMLModel(id);
            chai_1.assert.isUndefined(model);
        });
    });
    it('should return a model when a valid id is used', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const id = 'smt_new::smt_active';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const model = yield lytics.getSegmentMLModel(id);
            chai_1.assert.isDefined(model);
            chai_1.assert.isDefined(model.conf);
            chai_1.assert.isDefined(model.conf.source);
            chai_1.assert.isDefined(model.conf.source.slug_name);
            chai_1.assert.isDefined(model.conf.target);
            chai_1.assert.isDefined(model.conf.target.slug_name);
            chai_1.assert.equal(`${model.conf.source.slug_name}::${model.conf.target.slug_name}`, id);
        });
    });
});
describe('createSegmentMLModel', function () {
    it('should return a model when valid parameters are used', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const config = new types_1.CreateSegmentMLModelConfig();
            config.source = "all";
            config.target = "smt_active";
            config.use_scores = true;
            const model = yield lytics.createSegmentMLModel(config);
            chai_1.assert.isDefined(model);
        });
    });
    it('should throw an error if the source segment is too small', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const config = new types_1.CreateSegmentMLModelConfig();
            config.source = "smt_new";
            config.target = "smt_active";
            config.use_scores = true;
            try {
                const model = yield lytics.createSegmentMLModel(config);
                chai_1.assert.isTrue(false);
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
        });
    });
    it('should throw an error if the target segment is too small', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const config = new types_1.CreateSegmentMLModelConfig();
            config.source = "smt_active";
            config.target = "smt_new";
            config.use_scores = true;
            try {
                const model = yield lytics.createSegmentMLModel(config);
                chai_1.assert.isTrue(false);
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
        });
    });
});
describe('createSegmentMLModel', function () {
    it('should delete the model when the generation number is included', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const result = yield lytics.deleteSegmentMLModel('all::smt_active::2');
            chai_1.assert.isTrue(result);
        });
    });
    it('should delete the model when no generation number is included', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const result = yield lytics.deleteSegmentMLModel('all::smt_active');
            chai_1.assert.isTrue(result);
        });
    });
    it('should throw an error when the model does not exist', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const result = yield lytics.deleteSegmentMLModel('xxx::yyy');
            chai_1.assert.isFalse(result);
        });
    });
});
describe('getFragments', function () {
    it('should throw an error if parameters are missing', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            try {
                yield lytics.getFragments('', 'bbb', 'ccc');
                chai_1.assert.isTrue(false);
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
            try {
                yield lytics.getFragments('aaa', '', 'ccc');
                chai_1.assert.isTrue(false);
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
            try {
                yield lytics.getFragments('aaa', 'bbb', '');
                chai_1.assert.isTrue(false);
            }
            catch (err) {
                chai_1.assert.isTrue(true);
            }
        });
    });
    it('should return undefined if no matching fragments are found', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const fragments = yield lytics.getFragments('user', 'xxxxx', 'xxxxx');
            chai_1.assert.isUndefined(fragments);
        });
    });
    it('should return collection of user fragments if matching fragments are found', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const table = 'user';
            const field = 'email';
            const value = 'adam.conn@gmail.com';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const fragments = yield lytics.getFragments(table, field, value);
            chai_1.assert.isDefined(fragments);
            const entity = fragments.entity;
            chai_1.assert.isDefined(entity);
            chai_1.assert.equal(entity[field], value);
            const keys = fragments.keys;
            const key = keys.find(key => key.key === field);
            chai_1.assert.isDefined(key);
            chai_1.assert.equal(key.value, value);
        });
    });
    it('should return collection of content fragments if matching fragments are found', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const table = 'content';
            const field = 'hashedurl';
            const value = '8197029859537599926';
            const lytics = new LyticsClient_1.LyticsClient(apikey);
            const fragments = yield lytics.getFragments(table, field, value);
            chai_1.assert.isDefined(fragments);
            const entity = fragments.entity;
            chai_1.assert.isDefined(entity);
            chai_1.assert.equal(entity[field], value);
            const keys = fragments.keys;
            const key = keys.find(key => key.key === field);
            chai_1.assert.isDefined(key);
            chai_1.assert.equal(key.value, value);
        });
    });
});
describe('FragmentHashManager', function () {
    it('Hashes should match for 2 fragments with the same keys.', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const keyA = new types_1.FragmentKey();
            keyA.key = 'aaa';
            keyA.value = '111';
            const keyB = new types_1.FragmentKey();
            keyB.key = 'bbb';
            keyB.value = '222';
            const fragment1 = new types_1.Fragment();
            fragment1.key.push(keyA);
            fragment1.key.push(keyB);
            const fragment2 = new types_1.Fragment();
            fragment2.key.push(keyB);
            fragment2.key.push(keyA);
            const hash1 = types_1.FragmentHashManager.getHashForKey(fragment1.key);
            const hash2 = types_1.FragmentHashManager.getHashForKey(fragment2.key);
            chai_1.assert.equal(hash1, hash2);
        });
    });
});
//# sourceMappingURL=LyticsClient.spec.js.map